/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.

 Christian Herdtweck, Intra2net AG 2015
 Based on an example in Boost Documentation (by Christopher M. Kohlhoff)
 and adaptation by Guilherme M. Ferreira
 */

#include "icmp/icmpheader.h"
#include <sstream>
#include <boost/scoped_array.hpp>

IcmpHeader::IcmpHeader()
    : type( 0 )
    , code( 0 )
    , checksum( 0 )
{}

IcmpHeader::IcmpHeader(const uint8_t type_arg, const uint8_t code_arg)
    : type( type_arg )
    , code( code_arg )
    , checksum( 0 )
{}

uint16_t IcmpHeader::get_header_length() const { return 4; }
uint8_t  IcmpHeader::get_type()          const { return type; }
uint8_t  IcmpHeader::get_code()          const { return code; }
uint16_t IcmpHeader::get_checksum()      const { return checksum; }


std::istream& IcmpHeader::read(std::istream &is)
{
    boost::scoped_array<char> buf( new char[4] );
    is.read(buf.get(), 4);
    type = static_cast<uint8_t>(buf[0]);
    code = static_cast<uint8_t>(buf[1]);

    // first make the 2 checksum byte unsigned with same size
    uint8_t checksum_msb = static_cast<uint8_t>(buf[2]);
    uint8_t checksum_lsb = static_cast<uint8_t>(buf[3]);

    // now extend type, shift and add/or them
    checksum = static_cast<uint16_t>(
                            (static_cast<uint16_t>(checksum_msb) << 8)
                          |  static_cast<uint16_t>(checksum_lsb)        );
    // need the static_cast around the result since compiler casts everything
    // to int before doing | or + if types are smaller than int

    return is;
}

std::ostream& IcmpHeader::write(std::ostream &os) const
{
    uint8_t checksum_lsb = static_cast<uint8_t>(checksum & 0x00FF);
    uint8_t checksum_msb = static_cast<uint8_t>((checksum & 0xFF00) >> 8);
    return os << type << code << checksum_msb << checksum_lsb;
}


// code adapted from boost icmp example by Christopher M. Kohlhoff
// --> boost license?!?
void IcmpHeader::calc_checksum( const uint32_t body_checksum_part )
{
    uint32_t sum = (type << 8) + code;
    sum += body_checksum_part;

    sum = (sum >> 16) + (sum & 0xFFFF);
    sum += (sum >> 16);

    checksum = static_cast<uint16_t>( ~sum );
}


std::string IcmpHeader::to_string() const
{
    std::stringstream buf;
    buf << "[Icmp header type=" << static_cast<int>(type)
        << ",code=" << static_cast<int>(code)
    //    << " (cksum:" << std::showbase << std::hex << checksum << ")"
        << "]";
    return buf.str();
}


std::istream& operator>>(
        std::istream &is,
        IcmpHeader &header
)
{  return header.read( is );  }

std::ostream& operator<<(
        std::ostream &os,
        const IcmpHeader &header
)
{  return header.write( os );  }


// (created using vim -- the world's best text editor)

